from typing import Dict
from typing import Optional


def flatten(obj: Dict, seperator: Optional[str] = None) -> dict:
    """Flatten a dictionary.

    Time complexity: O(n * m) where n is the number of keys in the dictionary and m is the number of nested keys.

    Args:
        obj (Dict): The dictionary to flatten.
        seperator (Union[str, None]): The seperator to use.

    Returns:
        Dict: The flattened dictionary.
    """

    if not isinstance(seperator, str):
        seperator = '.'

    flattened = {}
    current = None
    nested = None

    for key in obj:
        current = obj[key]
        if isinstance(current, dict):
            nested = flatten(current, seperator)

            for _key in nested:
                flattened[key + seperator + _key] = nested[_key]
        else:
            flattened[key] = current

    return flattened



from typing import Dict

# Utilities
from .get import get


def exists(obj: Dict, path: str) -> bool:
    """
    Check if a value exists in a dictionary.

    Caution: If the value is None this function will return False even if the key exists.

    Time complexity: O(n)

    Args:
        obj: The dictionary to check.
        path: The path to the value.

    Returns:
        bool: True if the value exists, False otherwise.
    """

    return get(obj, path) is not None



from typing import Any
from typing import KeysView
from typing import List
from typing import Dict
from typing import Union
from typing import Optional


def transform(obj: Union[Dict,List], delimiter: Optional[str] = None) -> Union[Dict[str, Any],List]:
    """
    Transform unnested object with .-separated keys into a nested object.

    Time complexity: O(n * b)

    Args:
        obj (Union[Dict,List]): Object to transform.
        delimiter (Optional[str]): Delimiter to use. Defaults to '.'.

    Returns:
        Union[Dict[str, Any],List]: Transformed object.
    """

    if isinstance(obj, list):
        return [transform(item, delimiter) for item in obj]

    delimiter = delimiter or '.'

    pieces = None
    piecesLength = None
    piece = None
    current = None
    transformed: Dict[str, Any] = {}
    keys: KeysView[Any] = obj.keys()

    key: str
    for key in keys:

        if delimiter in key:
            pieces = key.split(delimiter)
            piecesLength = len(pieces)
            current = transformed

            for i in range(piecesLength):
                piece = pieces[i]

                if i != (piecesLength - 1) and piece not in current:
                    current[piece] = {}

                if i == (piecesLength - 1):
                    current[piece] = obj[key]

                current = current[piece]
                if current is None:
                    break

        else:
            transformed[key] = obj[key]

    return transformed



from typing import Any
from typing import Dict
from typing import Union


def get(obj: Dict, path: str, default: Any = None) -> Any:
    """
    Traverse object according to path, return value if found, else default.

    Time complexity: O(n)

    Args:
        obj: Object to traverse.
        path: Path to traverse.
        default: Default value to return if path not found.

    Returns:
        Any: Value at path if found, else default.
    """

    if obj is None or path is None:
        return default

    names = None

    if isinstance(path, str):
        names = path.split('.')
    elif isinstance(path, list):
        names = path
    else:
        raise TypeError('Path must be str or list')

    current: Union[Dict[str, Any], None] = obj
    for name in names:

        if isinstance(current, dict) and name in current:
            current = current[name]
        else:
            current = None

        if current is None:
            break

    return current if current is not None else default



from typing import Optional


def set_value(obj: dict, path: str, new_value: object, force: Optional[bool] = None) -> None:
    """
    Set a value in a nested object.

    Time Complexity: O(n) where n is the length of the path.

    Args:
        obj (Dict): The object to modify.
        path (str): The path to the value to modify.
        new_value (object): The new value to set.
        force (bool, optional): If True, will set the value even if the path does not exist. Defaults to None.
    """

    if not isinstance(obj, dict):
        raise TypeError('Value is not an object.')

    pieces = path.split('.') if isinstance(path, str) else path
    current = obj
    piece = None
    length = len(pieces)

    for index in range(length):
        piece = pieces[index]

        # Create namespace (object) where none exists.
        # If `force === true`, bruteforce the path without throwing errors.
        if (isinstance(current, dict) and piece not in current) or current[piece] is None or (not isinstance(current[piece], dict) and force):
            current[piece] = {}

        if index == (length - 1):
            # Set final value
            current[piece] = new_value
        else:
            # We do not overwrite existing path pieces by default
            if not isinstance(current[piece], dict):

                raise ValueError('Target key "{}" is not suitable for a nested value. (It is in use as non-object. Set `force` to `true` to override.)'.format(piece))

            # Traverse next in path
            current = current[piece]



from setuptools import setup  # type: ignore
from setuptools import find_packages

VERSION = '0.0.2'
DESCRIPTION = 'PyDottie is a library based on dottie.js written in Python 3.'

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

# Setting up the package
setup(
    name='pydottie',
    url='https://github.com/JoseNoriegaa/pydottie',
    version=VERSION,
    description=DESCRIPTION,
    long_description_content_type='text/markdown',
    long_description=long_description,
    author='josenoriega (Jose Noriega)',
    author_email='josenoriega723@gmail.com',
    install_requires=[],
    keywords=[
        'pydottie',
        'dottie',
        'dict',
        'dot path',
        'dot notation',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    project_urls={
        'Bug Tracker': 'https://github.com/pypa/sampleproject/issues',
    },
    package_dir={'': 'src'},
    packages=find_packages(where='src'),
    python_requires='>=3.6',
)



from typing import Any, Dict

# Utilities
from .get import get


def find(path: str, obj: Dict) -> Any:
    """
    Finds a value in a dictionary.

    Time complexity: O(n)

    Args:
        path: The path to the value.
        obj: The dictionary to search.

    Returns:
        Any: The value found.
    """

    return get(obj, path)



from typing import Dict
from typing import List
from typing import Optional


def paths(obj: Dict, prefixes: Optional[List[str]] = None) -> List[str]:
    """
    Gets the paths of the given object.

    Time Complexity: O(n * m) where n is the number of keys in the object and m is the number of nested keys.

    Args:
        obj (Dict): The object to get the paths of.
        prefixes (Union[List[str], None]): The prefixes to use.

    Returns:
        List[str]: The paths of the given object.
    """

    output: List[str] = []
    key = None

    prefixes = prefixes or []

    if not isinstance(obj, dict):
        raise TypeError('Paths was called with non-object argument.')

    for key, value in obj.items():

        if isinstance(value, dict) and value != None:
            output = output + paths(value, (prefixes + [key]))
        else:
            output.append('.'.join(prefixes + [key]))

    return output
